


#variable started 0

if {![info exists started]} {variable started 0}

if {$started != 0} {

    variable JS ""
    variable LogFlag

   if {$LogFlag == 1} {InsertLog "Last Updated: [clock format [clock seconds] -format "%B %d, %Y %H:%M:%S"]"}

    do_click

}

if {$started == 0} {
    set started 1

    variable ShowLogFlag 0

    variable LogText ""
    variable LogFlag 0
    variable LogCount 0

    variable JS ""

    variable HelpTopic "Tests"

    variable SyncCounter 0
    variable SyncPeriod 0

    global env
    variable FirmwareDir [file join $env(MIDASBASE) TclHttpd Html AGATADigitizer Bitstreams]

    variable sources {"core bitstream" "segment bitstream" "core control RAM" "segment control RAM"}
    variable srce [first $sources]

    variable filelist {"undefined"}
    variable filename [first $filelist]

    variable destinations {"core control RAM" "core module core" "core module segments" "segment control RAM" "segment module segments"}
    variable destination [first $destinations]

    variable Clocks {"External" "Internal"}
    variable Clock [first $Clocks]

    variable Stage
    variable bit
    variable ram

    variable FpgaN {"C1" "C2" "C0" "" "" "" "" "" "S1" "S2" "S3" "S4"}
    variable FpgaC
    array set FpgaC {}
    foreach fpga [list 2 0 1 8 9 10 11] {
       for {set Ch 1} {$Ch < 7} {incr Ch} {
           set FpgaC($fpga,$Ch) 0
       }
    }


   proc  show_variables {} {
        variable Digitizer
        variable FpgaList
        variable FpgaIDs
        variable Fpga
        variable srce
        variable destination
        variable filename
        variable SyncPeriod
        variable Clock


        set fpga [lindex $FpgaIDs [lsearch $FpgaList $Fpga]]

        InsertLog "Digitizer=$Digitizer; Fpga=$Fpga; fpga=$fpga"
        InsertLog "FpgaList=$FpgaList"
        InsertLog "source=$srce; destination=$destination; filename=$filename"
        InsertLog "SyncPeriod=$SyncPeriod"
        InsertLog "Clock=$Clock"
   }


      proc get_bitstreams {} {
        variable FirmwareDir
        variable filelist
        variable filename

        set z [catch {file isdirectory $FirmwareDir} m]
        if {$z != 0 || $m == 0} {
            Alert "Unable to access $FirmwareDir or $FirmwareDir is not a directory"
            return
        }

        set arg "-name \"*bit\" "
        set z [catch {eval exec find $FirmwareDir $arg} m]

        if {$z != 0} {
            Alert "Unable to obtain firmware files: $m"
            return
        }

        set filelist {}
        foreach item $m {
            if {[string length $item] == 0} {continue}
            lappend filelist [file tail $item]
        }

        if {[llength $filelist] == 0} {set filename "undefined"} else {set filename [first $filelist]}

     }

     proc do_download {} {
        variable FirmwareDir
        variable srce
        variable destination
        variable filename
        variable Digitizer
        variable Stage
        variable bit
        variable ram
        upvar #0 AGATADigitizerAccessServer_XportList XportList

#       if {$ModuleName == "undefined"} {Alert "please select module to program!"; return}

       set IPAddress [first [split [set XportList($Digitizer)] :]]
       set IPPort [second [split [set XportList($Digitizer)] :]]

       set FirmwareFile [file join $FirmwareDir $filename]

       if {$Stage == 0} {

#    check value source/destination combination

          switch $srce {
              "core bitstream" {
                  set bit 1
                  switch $destination {
                     "core control RAM" -
                     "core module core"     {set OK 1}
                     default         {set OK 0}
                  }
              }
              "segment bitstream" {
                  set bit 1
                  switch $destination {
                     "core module core"     {set OK 0}
                     default         {set OK 1}
                  }
              }
              "core control RAM" {
                  set bit 0
                  switch $destination {
                     "core module segments" -
                     "core module core"     {set OK 1}
                     default         {set OK 0}
                  }
              }
              "segment control RAM" {
                  set bit 0
                  switch $destination {
                     "segment module segments"  {set OK 1}
                     default                    {set OK 0}
                  }
              }
              default {set OK 0}
          }

          switch  $destination {
              "core control RAM"  -
              "segment control RAM" {
                  set ram 1
              }
              default {set ram 0}
          }
          
          if {$OK == 0} {Alert "source = $srce and destination = $destination is an invalid combination"; return}

          if {$bit == 1} {
              if {$filename == "undefined"} {Alert "please select a firmware file to use!"; return}
              Confirm Tests "Load firmware $FirmwareFile into $destination for Digitizer $Digitizer (Address $IPAddress; Port $IPPort)\n\nThis will take some time\nTime estimated 2+ minutes"
              return
          }
          if {$bit == 0} {
              Confirm Tests "Load $srce into $destination for Digitizer $Digitizer (Address $IPAddress; Port $IPPort)\n"
              return
          }
       }

       if {$Stage == 1 && $bit == 1} {

#    first stage is to copy a bitstream file into digitizer RAM

          do_close

          switch -glob $srce {

            "core bitstream"  {
               set cmd 0x20
               set file $FirmwareFile
               set cmd "[file join $FirmwareDir loader.exe] -n $IPAddress -p $IPPort -f $FirmwareFile -c $cmd"
               InsertLog "executing $cmd\\n"
               set z [catch {eval exec $cmd} m]
               InsertLog "Action completed with rc = $z and response \\n$m"
               Alert "finished"
             }
            "segment bitstream"  {
               set cmd 0xa0
               set file $FirmwareFile
               set cmd "[file join $FirmwareDir loader.exe] -n $IPAddress -p $IPPort -f $FirmwareFile -c $cmd"
               InsertLog "executing $cmd\\n"
               set z [catch {eval exec $cmd} m]
               InsertLog "Action completed with rc = $z and response \\n$m"
               Alert "finished"
             }
             default {}
          }

          if {$ram == 0} {
              Confirm Tests "Loaded source bitstream. Now ready to program destination Virtex"
          } else {
              Confirm Tests "Loaded source bitstream."
          }
          return

       }
       if {$Stage == 1 && $bit == 0} {

#     source is module RAM - so skip this stage

          Confirm Tests "Program source is control RAM. Are you sure you have previously loaded a suitable bitstream file?"

       }

      if {$Stage == 2 && $ram == 0} {

          do_close
       
          switch -glob $destination {

            "core module segments"  {
               set cmd 0x00
               set action 3
               set cmd "[file join $FirmwareDir loader.exe] -n $IPAddress -p $IPPort -c $cmd -a $action"
               InsertLog "executing $cmd\\n"
               set z [catch {eval exec $cmd} m]
               InsertLog "Action completed with rc = $z and response \\n$m"
               Alert "finished"
             }
            "core module core"  {
               set cmd 0x00
               set action 4
               set cmd "[file join $FirmwareDir loader.exe] -n $IPAddress -p $IPPort -c $cmd -a $action"
               InsertLog "executing $cmd\\n"
               set z [catch {eval exec $cmd} m]
               InsertLog "Action completed with rc = $z and response \\n$m"
               Alert "finished"
             }
            "segment module segments"  {
               set cmd 0x80
               set action 15
               set cmd "[file join $FirmwareDir loader.exe] -n $IPAddress -p $IPPort -c $cmd -a $action"
               InsertLog "executing $cmd\\n"
               set z [catch {eval exec $cmd} m]
               InsertLog "Action completed with rc = $z and response \\n$m"
               Alert "finished"
             }
             default {}
          }

          Confirm Tests "Loaded destination"
          return

      }

      if {$Stage == 3} {

          Alert "All finished"

      }
    
   }
   
   proc set_sync {} {
        variable SyncPeriod

        variable FpgaList
        variable FpgaIDs
        variable Fpga

        set fpga [lindex $FpgaIDs [lsearch $FpgaList $Fpga]]
 
        if {[llength $FpgaIDs] == 0 || $fpga != 2} {

           InsertLog "Select Core ADC card"
           return
        }

        Set_Register 0xb0 [expr $SyncPeriod & 0x0000ffff]
        Set_Register 0xb1 [expr ($SyncPeriod >> 16) & 0x0000ffff]
   }

   proc get_sync {opt} {
        variable SyncPeriod
        variable SyncCounter

        variable Digitizer
        variable FpgaList
        variable FpgaIDs
        variable Fpga

        if {$Fpga == "Use Probe Hardware"} {return}

        set FpgaBAK $Fpga
        set Fpga "Core: Virtex for core ADCs"

        set LSW [Get_Register 0xb0]
        set MSW [Get_Register 0xb1]

        set SyncPeriod [format 0x%08x [expr ($MSW << 16) + $LSW]]
        set SyncCounter [Get_Register 0xb2]

        set Fpga $FpgaBAK
   }

    proc get_clock {} {
        variable Digitizer
        variable FpgaList
        variable FpgaIDs
        variable Fpga
        global errorCode errorInfo
        variable Clock

        if {$Fpga == "Use Probe Hardware"} {return}

        set fpga [lindex $FpgaIDs [lsearch $FpgaList "Core: Main Board"]]

        set cmd 14     ;# read status registers command
        set Length 6   ;# returns 6 bytes
        set rc [catch {interp eval "" ::namespace inscope ::urn:AGATADigitizerAccessServer do_Read $Digitizer $fpga $cmd A16 $Length} m]
        if {$rc == 0} {
            set reg0 [first $m]
            if {[expr $reg0 && 1] == 0} {set Clock "External"} else {set Clock "Internal"}
        } else {
            InsertLog "Write returned with an error\n$m\n$errorCode\n$errorInfo"
        }
    }

   proc sync_err {} {
        global errorCode errorInfo
        variable Digitizer
        variable FpgaList
        variable FpgaIDs
        variable Fpga
        variable FpgaN
        variable FpgaC

        foreach fpga [list 2 0 1 8 9 10 11] {
           for {set Ch 1} {$Ch < 7} {incr Ch} {
              if {$fpga == 2 && $Ch > 2} {break}
              set addr [expr ($Ch * 16) + 11]
              set rc [catch {interp eval "" ::namespace inscope ::urn:AGATADigitizerAccessServer do_Get $Digitizer $fpga $addr A16} m]
              if {$rc != 0} {
                   InsertLog "Read returned with an error\n$m\n$errorCode\n$errorInfo"
                   return
              }
              set v$fpga$Ch $m
              set d$fpga$Ch [expr $m - [set FpgaC($fpga,$Ch)]]
              set FpgaC($fpga,$Ch) $m
           }
        }

        InsertLog "Counters"

        foreach fpga [list 2 0 1 8 9 10 11] {
           if {$fpga == 2} {
               InsertLog "[lindex $FpgaN $fpga]: Ch1=[set v[set fpga]1]; Ch2=[set v[set fpga]2]"
           } else {
               InsertLog "[lindex $FpgaN $fpga]: Ch1=[set v[set fpga]1]; Ch2=[set v[set fpga]2]; Ch3=[set v[set fpga]3]; Ch4=[set v[set fpga]4]; Ch5=[set v[set fpga]5]; Ch6=[set v[set fpga]6]"
           }
        }

        InsertLog "Differences"

        foreach fpga [list 2 0 1 8 9 10 11] {
           if {$fpga == 2} {
               InsertLog "[lindex $FpgaN $fpga]: Ch1=[set d[set fpga]1]; Ch2=[set d[set fpga]2]"
           } else {
               InsertLog "[lindex $FpgaN $fpga]: Ch1=[set d[set fpga]1]; Ch2=[set d[set fpga]2]; Ch3=[set d[set fpga]3]; Ch4=[set d[set fpga]4]; Ch5=[set d[set fpga]5]; Ch6=[set d[set fpga]6]"
           }
        }
   }

   proc do_samwise {vl} {
        variable Digitizer
        variable Fpga
        variable Address
        variable AddressModifier
        variable Data
        variable Length

        variable FpgaList
        variable FpgaIDs

        global errorCode errorInfo

        set fpga [lindex $FpgaIDs [lsearch $FpgaList "Core: Main Board"]]

        set Address 0x0c1e     ;#   enable/disable segment Xport
        if {$vl == 1} {set Data 0x0100}  ;#   enable segment Xport
        if {$vl == 2} {set Data 0x0000}  ;#   disable segment Xport
        set AddressModifier A16

        set rc [catch {interp eval "" ::namespace inscope ::urn:AGATADigitizerAccessServer do_Set $Digitizer $fpga $Address $AddressModifier 1 $Data} m]
        if {$rc != 0} {
            InsertLog "Set returned with an error\n$m\n$errorCode\n$errorInfo"
        } else {
            if {$vl == 1} {
                do_close          ;#   release core Xport
                InsertLog "Core Xport interface released; Segment Xport interface enabled. You can now launch Samwise"
            }
            if {$vl == 2} {
                InsertLog "Segment Xport interface disabled"
            }
        }
   }

    proc do_showstatus {} {
        variable Digitizer 
        variable Fpga
        variable FpgaList 
        variable FpgaIDs 
        global errorCode errorInfo

        set fpga [lindex $FpgaIDs [lsearch $FpgaList $Fpga]]
        if {$fpga < 8} then {set fpga 3} else {set fpga 12}    ;# select main board

        set cmd 14     ;# read status registers command
        set Length 6   ;# returns 6 bytes
        set rc [catch {interp eval "" ::namespace inscope ::urn:AGATADigitizerAccessServer do_Read $Digitizer $fpga $cmd A16 $Length} m]
        if {$rc == 0} {
            InsertLog "status registers: $m"
        } else {
            InsertLog "Write returned with an error\n$m\n$errorCode\n$errorInfo"
        }
    }

    proc do_showfmwre {} {
        variable Digitizer
        variable Fpga
        variable FpgaList 
        variable FpgaIDs 
        global errorCode errorInfo

           foreach fpga [list 2] {

             set fpgaId  [lindex $FpgaList 1]

              set Addr 6
              set rc [catch {interp eval "" ::namespace inscope ::urn:AGATADigitizerAccessServer do_Get $Digitizer $fpga $Addr A16} m]
              if {$rc == 0} {
                  InsertLog "Digitizer $Digitizer $fpgaId: core firmware version: ([format "0x%x" $m])"
              } else {
                  InsertLog "Digitizer $Digitizer $fpgaId: access returned with an error\n$m\n$errorCode\n$errorInfo"
              }
           }

           set i 2

           foreach fpga [list 0 1 8 9 10 11] {

              set fpgaId  [lindex $FpgaList $i]
              incr i
              if {$fpga == 1} {incr i}

              set Addr 15
              set rc [catch {interp eval "" ::namespace inscope ::urn:AGATADigitizerAccessServer do_Get $Digitizer $fpga $Addr A16} m]
              if {$rc == 0} {
                  InsertLog "Digitizer $Digitizer $fpgaId: segment firmware version: ([format "0x%x" $m])"
              } else {
                  InsertLog "Digitizer $Digitizer $fpgaId: access returned with an error\n$m\n$errorCode\n$errorInfo"
              }
           }
    }

    proc do_showallstatus {} {
        variable DigitizerList
        variable Fpga
        variable FpgaList 
        variable FpgaIDs 
        global errorCode errorInfo

        foreach digitizer $DigitizerList {

           set fpga 3    ;# select core main board
           set cmd 14     ;# read status registers command
           set Length 6   ;# returns 6 bytes
           set rc [catch {interp eval "" ::namespace inscope ::urn:AGATADigitizerAccessServer do_Read $digitizer $fpga $cmd A16 $Length} m]
           if {$rc == 0} {
              InsertLog "Digitizer $digitizer: core status registers: $m"
           } else {
              InsertLog "Digitizer $digitizer: access error"
              break
           }

           set fpga 12    ;# select segment main board
           set cmd 14     ;# read status registers command
           set Length 6   ;# returns 6 bytes
           set rc [catch {interp eval "" ::namespace inscope ::urn:AGATADigitizerAccessServer do_Read $digitizer $fpga $cmd A16 $Length} m]
           if {$rc == 0} {
              InsertLog "Digitizer $digitizer: segment status registers: $m"
           } else {
              InsertLog "Digitizer $digitizer: access error"
              break
           }
        }
    }

    proc do_showallfmwre {} {
        variable DigitizerList
        variable Fpga
        variable FpgaList 
        variable FpgaIDs 
        global errorCode errorInfo

        foreach digitizer $DigitizerList {

           foreach fpga [list 2] {

              set Addr 6
              set rc [catch {interp eval "" ::namespace inscope ::urn:AGATADigitizerAccessServer do_Get $digitizer $fpga $Addr A16} m]
              if {$rc == 0} {
                  InsertLog "Digitizer $digitizer.$fpga: core firmware version: ([format "0x%x" $m])"
              } else {
                  InsertLog "Digitizer $digitizer.$fpga: access error"
                  break
              }
           }

           foreach fpga [list 0 1 8 9 10 11] {

              set Addr 15
              set rc [catch {interp eval "" ::namespace inscope ::urn:AGATADigitizerAccessServer do_Get $digitizer $fpga $Addr A16} m]
              if {$rc == 0} {
                  InsertLog "Digitizer $digitizer.$fpga: segment firmware version: ([format "0x%x" $m])"
              } else {
                  InsertLog "Digitizer $digitizer.$fpga: access error"
                  break
              }
           }
        }
    }

   proc  do_click {}  {
       variable cgidata
       variable started
       variable LogText
       variable ShowLogFlag
       variable LogFlag
       variable Stage
       variable HelpTopic

       set cgidata [ncgi::nvlist]

       if {$LogFlag == 1} {InsertLog "cgidata= ([llength $cgidata]) $cgidata"}

       if {[llength $cgidata] == 0} then {return}

       if {[llength $cgidata] < 2} then {
          InsertLog "CGI data is too short: $cgidata"
       } else {
          if {[lindex $cgidata 0] != "Widget"} then {
             InsertLog "Invalid CGI data: $cgidata"
          } else {

           set w [first [lindex $cgidata 1]]
           set vl [second [lindex $cgidata 1]]

            switch -glob $w {

                RESET    {set started 0}
                   CLEAR    {EmptyLog}
                   ELOG     {PrintLog}
                SELECT   {}
                RELOAD   {get_clock}
                VARIABLES {show_variables}

                HIDE     {set ShowLogFlag 0}
                SHOW     {set ShowLogFlag 1}
                   LOGON    {set LogFlag 1}
                   LOGOFF   {set LogFlag 0}

                   HELP     {
                       switch $vl {
                          0  {set HelpTopic "Tests"}
                          1  {set HelpTopic "Program"}
                          default {set HelpTopic "Agata"}
                       }
                       do_help
                   }
                
                PROBE    {click-probe}

                FPGA       {variable Fpga; set Fpga [string range [lindex $cgidata 1] 5 end]; get_sync 1; get_clock}
                DIGITIZER  {selectDigitizer $vl; get_sync 1; get_clock}

                SYNC       {variable SyncPeriod; set SyncPeriod [second [lindex $cgidata 1]]; set_sync}

                FILE         {variable filename; set filename $vl}
                DESTINATION  {variable destination; set destination [string range [lindex $cgidata 1] 12 end]}
                SOURCE       {variable srce; set srce [string range [lindex $cgidata 1] 7 end]}

                UPDTSYNC  {get_sync 0}
                SYNCERR   {sync_err}

                SHUTDOWN   {do_shutdown}
                DOSHOW    {
                    switch $vl {
                       0     {do_showstatus}
                       1     {do_showfmwre}
                       2     {do_showallstatus}
                       3     {do_showallfmwre}
                    }
                }

                CLOCK      {
                    variable Clock
                    set Clock $vl
                    switch $vl {
                       Internal   {do_clock 0x0100}
                       External   {do_clock 0x0000}
                    }
                }

                CLOSEDOWN  {do_close}

                DOWNLOAD   {set Stage 0; do_download}
                CONFIRM    {incr Stage; do_download}

                SAMWISE     {do_samwise $vl}

                MainLog {}
                undefined {}
                default {InsertLog "unexpected CGI data:  $cgidata"}
            }

            
         }
      }
   }   ;# end of do_clock

   EnumerateDigitizers
   selectDigitizer [first $DigitizerList]

   get_bitstreams
   get_clock

}   ;# end of if {$started == 0} 

